@@ -17,7 +17,6 @@ from hashlib import md5
 from lollypop.art_base import BaseArt
 from lollypop.art_album import AlbumArt
 from lollypop.art_artist import ArtistArt
-from lollypop.art_radio import RadioArt
 from lollypop.art_downloader import DownloaderArt
 from lollypop.logger import Logger
 from lollypop.define import CACHE_PATH, ALBUMS_WEB_PATH, ALBUMS_PATH
@@ -27,7 +26,7 @@ from lollypop.utils import emit_signal
 from lollypop.utils_file import create_dir, remove_oldest
 
 
-class Art(BaseArt, AlbumArt, ArtistArt, RadioArt, DownloaderArt):
+class Art(BaseArt, AlbumArt, ArtistArt, DownloaderArt):
     """
         Global artwork manager
     """
@@ -39,7 +38,6 @@ class Art(BaseArt, AlbumArt, ArtistArt, RadioArt, DownloaderArt):
         BaseArt.__init__(self)
         AlbumArt.__init__(self)
         ArtistArt.__init__(self)
-        RadioArt.__init__(self)
         DownloaderArt.__init__(self)
         # Move old store
         # FIXME: Remove this later
