@@ -14,8 +14,10 @@ from gi.repository import Gio, GLib
 
 from gettext import gettext as _
 
-from lollypop.define import App
-from lollypop.utils import tracks_to_albums, emit_signal
+from lollypop.define import App, ViewType, Type
+from lollypop.utils_album import tracks_to_albums
+from lollypop.utils import get_default_storage_type, emit_signal
+from lollypop.utils import get_network_available
 from lollypop.objects_track import Track
 from lollypop.objects_album import Album
 
@@ -42,6 +44,21 @@ class PlaybackMenu(Gio.Menu):
 #######################
 # PROTECTED           #
 #######################
+    def _set_radio_action(self, artist_ids):
+        """
+            Set radio action
+            @param artist_ids as [int]
+        """
+        radio_action = Gio.SimpleAction(name="radio_action_collection")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Play a radio"),
+                                     "app.radio_action_collection")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        self.append_item(menu_item)
+
     def _set_playback_actions(self):
         """
             Setup playback actions
@@ -64,6 +81,115 @@ class PlaybackMenu(Gio.Menu):
         menu_item.set_attribute_value("close", GLib.Variant("b", True))
         self.append_item(menu_item)
 
+#######################
+# PRIVATE             #
+#######################
+    def __on_radio_action_activate(self, action, variant, artist_ids):
+        """
+            Play a radio from storage type
+            @param Gio.SimpleAction
+            @param GLib.Variant
+            @param artist_ids as [int]
+        """
+        App().player.play_radio_from_collection(artist_ids)
+
+
+class RadioPlaybackMenu(Gio.Menu):
+    """
+        Allow to play a radio from artist ids
+    """
+
+    def __init__(self, artist_ids):
+        """
+            Init menu
+            @param artist_ids as [int]
+        """
+        Gio.Menu.__init__(self)
+        section = Gio.Menu()
+        radio_action = Gio.SimpleAction(name="radio_action_collection")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Related tracks"),
+                                     "app.radio_action_collection")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_loved")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Loved tracks"),
+                                     "app.radio_action_loved")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_populars")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        menu_item = Gio.MenuItem.new(_("Popular tracks"),
+                                     "app.radio_action_populars")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        self.append_section(_("From collection"), section)
+        section = Gio.Menu()
+        radio_action = Gio.SimpleAction(name="radio_action_deezer")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        radio_action.set_enabled(get_network_available("DEEZER"))
+        menu_item = Gio.MenuItem.new(_("Deezer"),
+                                     "app.radio_action_deezer")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_lastfm")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        radio_action.set_enabled(get_network_available("LASTFM"))
+        menu_item = Gio.MenuItem.new(_("Last.fm"),
+                                     "app.radio_action_lastfm")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        radio_action = Gio.SimpleAction(name="radio_action_spotify")
+        App().add_action(radio_action)
+        radio_action.connect("activate",
+                             self.__on_radio_action_activate,
+                             artist_ids)
+        radio_action.set_enabled(get_network_available("SPOTIFY"))
+        menu_item = Gio.MenuItem.new(_("Spotify"),
+                                     "app.radio_action_spotify")
+        menu_item.set_attribute_value("close", GLib.Variant("b", True))
+        section.append_item(menu_item)
+        self.append_section(_("From the Web"), section)
+
+#######################
+# PRIVATE             #
+#######################
+    def __on_radio_action_activate(self, action, variant, artist_ids):
+        """
+            Play a radio from storage type
+            @param Gio.SimpleAction
+            @param GLib.Variant
+            @param artist_ids as [int]
+        """
+        if action.get_name() == "radio_action_collection":
+            App().player.play_radio_from_collection(artist_ids)
+        elif action.get_name() == "radio_action_spotify":
+            App().player.play_radio_from_spotify(artist_ids)
+        elif action.get_name() == "radio_action_lastfm":
+            App().player.play_radio_from_lastfm(artist_ids)
+        elif action.get_name() == "radio_action_deezer":
+            App().player.play_radio_from_deezer(artist_ids)
+        elif action.get_name() == "radio_action_loved":
+            App().player.play_radio_from_loved(artist_ids)
+        elif action.get_name() == "radio_action_populars":
+            App().player.play_radio_from_populars(artist_ids)
+
 
 class PlaylistPlaybackMenu(Gio.Menu):
     """
@@ -95,13 +221,18 @@ class PlaylistPlaybackMenu(Gio.Menu):
             @param playlist_id as int
         """
         if App().playlists.get_smart(playlist_id):
-            tracks = []
             request = App().playlists.get_smart_sql(playlist_id)
-            for track_id in App().db.execute(request):
-                tracks.append(Track(track_id))
+            # We need to inject skipped/storage_type
+            storage_type = get_default_storage_type()
+            split = request.split("ORDER BY")
+            split[0] += " AND loved != %s" % Type.NONE
+            split[0] += " AND tracks.storage_type&%s " % storage_type
+            track_ids = App().db.execute("ORDER BY".join(split))
+            albums = tracks_to_albums(
+                [Track(track_id) for track_id in track_ids])
         else:
             tracks = App().playlists.get_tracks(playlist_id)
-        albums = tracks_to_albums(tracks)
+            albums = tracks_to_albums(tracks)
         App().player.play_albums(albums)
 
 
@@ -110,21 +241,30 @@ class ArtistPlaybackMenu(PlaybackMenu):
         Contextual menu for an artist
     """
 
-    def __init__(self, artist_id):
+    def __init__(self, artist_id, storage_type):
         """
             Init menu
             @param artist id as int
+            @param storage_type as StorageType
         """
         PlaybackMenu.__init__(self)
         self.__artist_id = artist_id
+        self.__storage_type = storage_type
         play_action = Gio.SimpleAction(name="artist_play_action")
         App().add_action(play_action)
-        play_action.connect("activate", self.__play)
+        play_action.connect("activate", self.__on_play_action_activate)
         menu_item = Gio.MenuItem.new(_("Play this artist"),
                                      "app.artist_play_action")
         menu_item.set_attribute_value("close", GLib.Variant("b", True))
         self.append_item(menu_item)
         self._set_playback_actions()
+        if get_network_available("SPOTIFY") or\
+                get_network_available("LASTFM") or\
+                get_network_available("DEEZER"):
+            submenu = RadioPlaybackMenu([artist_id])
+            self.append_submenu(_("Play a radio"), submenu)
+        else:
+            self._set_radio_action([artist_id])
 
     @property
     def in_player(self):
@@ -132,7 +272,8 @@ class ArtistPlaybackMenu(PlaybackMenu):
             True if current object in player
             return bool
         """
-        album_ids = App().albums.get_ids([self.__artist_id], [])
+        album_ids = App().albums.get_ids([], [self.__artist_id],
+                                         self.__storage_type, False)
         return set(App().player.album_ids) & set(album_ids) == set(album_ids)
 
 #######################
@@ -159,7 +300,7 @@ class ArtistPlaybackMenu(PlaybackMenu):
 #######################
 # PRIVATE             #
 #######################
-    def __play(self, action, variant):
+    def __on_play_action_activate(self, action, variant):
         """
             Play albums
             @param Gio.SimpleAction
@@ -218,6 +359,8 @@ class GenrePlaybackMenu(PlaybackMenu):
             @param GLib.Variant
         """
         album_ids = self.__get_album_ids()
+        if App().player.current_track.album.id in album_ids:
+            App().player.skip_album()
         App().player.remove_album_by_ids(album_ids)
 
 #######################
@@ -228,8 +371,13 @@ class GenrePlaybackMenu(PlaybackMenu):
             Get album ids for genre
             @return [int]
         """
-        album_ids = App().albums.get_compilation_ids([self.__genre_id], True)
-        album_ids += App().albums.get_ids([], [self.__genre_id], True)
+        storage_type = get_default_storage_type()
+        album_ids = App().albums.get_compilation_ids([self.__genre_id],
+                                                     storage_type,
+                                                     False)
+        album_ids += App().albums.get_ids([self.__genre_id], [],
+                                          storage_type,
+                                          False)
         return album_ids
 
     def __play(self, action, variant):
@@ -292,6 +440,8 @@ class DecadePlaybackMenu(PlaybackMenu):
             @param GLib.Variant
         """
         album_ids = self.__get_album_ids()
+        if App().player.current_track.album.id in album_ids:
+            App().player.skip_album()
         App().player.remove_album_by_ids(album_ids)
 
 #######################
@@ -302,10 +452,13 @@ class DecadePlaybackMenu(PlaybackMenu):
             Get album ids for decade
             @return [int]
         """
+        storage_type = get_default_storage_type()
         album_ids = []
         for year in self.__years:
-            album_ids += App().albums.get_compilations_for_year(year)
-            album_ids += App().albums.get_albums_for_year(year)
+            album_ids += App().albums.get_compilation_ids_for_year(
+                year, storage_type, False)
+            album_ids += App().albums.get_ids_for_year(
+                year, storage_type, False)
         return album_ids
 
     def __play(self, action, variant):
@@ -324,21 +477,30 @@ class AlbumPlaybackMenu(PlaybackMenu):
         Contextual menu for an album
     """
 
-    def __init__(self, album):
+    def __init__(self, album, view_type):
         """
             Init album menu
             @param album as Album
+            @param view_type as ViewType
         """
         PlaybackMenu.__init__(self)
         self.__album = album
-        play_action = Gio.SimpleAction(name="album_play_action")
-        App().add_action(play_action)
-        play_action.connect("activate", self.__play)
-        menu_item = Gio.MenuItem.new(_("Play this album"),
-                                     "app.album_play_action")
-        menu_item.set_attribute_value("close", GLib.Variant("b", True))
-        self.append_item(menu_item)
-        self._set_playback_actions()
+        if not view_type & ViewType.BANNER:
+            play_action = Gio.SimpleAction(name="play_action")
+            App().add_action(play_action)
+            play_action.connect("activate", self.__on_play_action_activate)
+            menu_item = Gio.MenuItem.new(_("Play this album"),
+                                         "app.play_action")
+            menu_item.set_attribute_value("close", GLib.Variant("b", True))
+            self.append_item(menu_item)
+            self._set_playback_actions()
+        if get_network_available("SPOTIFY") or\
+                get_network_available("LASTFM") or\
+                get_network_available("DEEZER"):
+            submenu = RadioPlaybackMenu(album.artist_ids)
+            self.append_submenu(_("Play a radio"), submenu)
+        else:
+            self._set_radio_action(album.artist_ids)
 
     @property
     def in_player(self):
@@ -365,18 +527,20 @@ class AlbumPlaybackMenu(PlaybackMenu):
             @param Gio.SimpleAction
             @param GLib.Variant
         """
+        if App().player.current_track.album.id == self.__album.id:
+            App().player.skip_album()
         App().player.remove_album_by_id(self.__album.id)
 
 #######################
 # PRIVATE             #
 #######################
-    def __play(self, action, variant):
+    def __on_play_action_activate(self, action, variant):
         """
             Play album
             @param Gio.SimpleAction
             @param GLib.Variant
         """
-        App().player.play_album(self.__album)
+        App().player.play_album(self.__album.clone(True))
 
 
 class TrackPlaybackMenu(PlaybackMenu):
@@ -384,15 +548,17 @@ class TrackPlaybackMenu(PlaybackMenu):
         Contextual menu for tracks
     """
 
-    def __init__(self, track):
+    def __init__(self, track, view_type):
         """
             Init track menu
             @param track as Track
+            @param view_type as ViewType
         """
         PlaybackMenu.__init__(self)
         self.__track = track
-        self._set_playback_actions()
-        self.__set_queue_actions()
+        if not view_type & ViewType.TOOLBAR:
+            self._set_playback_actions()
+            self.__set_queue_actions()
         self.__set_stop_after_action()
 
     @property
@@ -416,23 +582,17 @@ class TrackPlaybackMenu(PlaybackMenu):
             @param Gio.SimpleAction
             @param GLib.Variant
         """
-        albums = App().player.albums
-        # If album last in list, merge
-        if albums and albums[-1].id == self.__track.album.id:
-            albums[-1].append_track(self.__track)
-            App().player.set_next()
-        # Add album with only one track
+        album = Album(self.__track.album.id)
+        album.set_tracks([self.__track])
+        App().player.add_album(album)
+        if App().player.is_playing:
+            App().player.add_album(album)
         else:
-            album = Album(self.__track.album.id)
-            album.set_tracks([self.__track])
-            if App().player.is_playing:
-                App().player.add_album(album)
-            else:
-                App().player.play_album(album)
+            App().player.play_album(album)
 
     def _remove_from_playback(self, action, variant):
         """
-            Delete track id from playback
+            Delete track from playback
             @param Gio.SimpleAction
             @param GLib.Variant
         """
@@ -441,10 +601,8 @@ class TrackPlaybackMenu(PlaybackMenu):
                 if self.__track.id in album.track_ids:
                     index = album.track_ids.index(self.__track.id)
                     track = album.tracks[index]
-                    album.remove_track(track)
+                    App().player.remove_track_from_album(track, album)
                     break
-        App().player.set_next()
-        App().player.set_prev()
 
 #######################
 # PRIVATE             #
