@@ -16,8 +16,8 @@ from gettext import gettext as _
 
 from lollypop.define import App, ArtSize, ArtBehaviour, Type, StorageType
 from lollypop.logger import Logger
-from lollypop.utils import get_network_available, sql_escape, get_youtube_dl
-from lollypop.utils_artist import ArtistProvider
+from lollypop.utils import sql_escape
+from lollypop.utils_file import get_youtube_dl
 
 
 class ArtistRow(Gtk.ListBoxRow):
@@ -46,7 +46,7 @@ class ArtistRow(Gtk.ListBoxRow):
         self.__artwork = Gtk.Image.new()
         self.__artwork.set_size_request(ArtSize.SMALL,
                                         ArtSize.SMALL)
-        if App().art.artist_artwork_exists(artist_name)[0]:
+        if App().art.get_artist_artwork_path(artist_name) is not None:
             App().art_helper.set_artist_artwork(artist_name,
                                                 ArtSize.SMALL,
                                                 ArtSize.SMALL,
@@ -93,7 +93,8 @@ class ArtistRow(Gtk.ListBoxRow):
                 return
             self.__cover_data = data
             scale_factor = self.get_scale_factor()
-            App().art.add_artist_artwork(self.__artist_name, data, True)
+            App().art.add_artist_artwork(self.__artist_name, data,
+                                         StorageType.EPHEMERAL)
             App().art_helper.set_artist_artwork(self.__artist_name,
                                                 ArtSize.SMALL,
                                                 ArtSize.SMALL,
@@ -117,6 +118,9 @@ class ArtistRow(Gtk.ListBoxRow):
                                                    self.__cancellable,
                                                    self.__on_uri_content)
                 self.__cover_uri = None
+            # Cache for later usage
+            else:
+                App().art.cache_artist_artwork(self.__artist_name)
             self.__artwork.get_style_context().add_class("circle-icon")
             self.__artwork.set_from_icon_name("avatar-default-symbolic",
                                               Gtk.IconSize.INVALID)
@@ -133,24 +137,27 @@ class SimilarsMenu(Gtk.Bin):
         A popover with similar artists
     """
 
-    def __init__(self):
+    def __init__(self, artist_id):
         """
             Init popover
+            @param artist_id as int
         """
         Gtk.Bin.__init__(self)
         (path, env) = get_youtube_dl()
         self.__show_all = path is not None
         self.__added = []
-        self.__artist = ""
+        self.__artist_id = artist_id
         self.__cancellable = Gio.Cancellable()
+        self.connect("map", self.__on_map)
         self.connect("unmap", self.__on_unmap)
         self.__stack = Gtk.Stack.new()
         self.__stack.show()
         self.__stack.set_transition_type(Gtk.StackTransitionType.CROSSFADE)
         self.__stack.set_transition_duration(200)
-        self.__spinner = Gtk.Spinner.new()
-        self.__spinner.show()
-        self.__spinner.start()
+        self.__label = Gtk.Label.new(_("Loading…"))
+        self.__label.get_style_context().add_class("bold")
+        self.__label.get_style_context().add_class("dim-label")
+        self.__label.show()
         self.__listbox = Gtk.ListBox()
         self.__listbox.get_style_context().add_class("trackswidget")
         self.__listbox.set_vexpand(True)
@@ -159,50 +166,21 @@ class SimilarsMenu(Gtk.Bin):
         self.__listbox.connect("row-activated", self.__on_row_activated)
         self.__listbox.set_sort_func(self.__sort_func)
         self.__listbox.show()
-        self.__stack.add(self.__spinner)
+        self.__stack.add(self.__label)
         self.__stack.add(self.__listbox)
-        label = Gtk.Label.new(_("No results"))
-        label.get_style_context().add_class("bold")
-        label.get_style_context().add_class("dim-label")
-        label.show()
-        self.__stack.add_named(label, "no-results")
         self.add(self.__stack)
 
-    def populate(self, artist_id):
+    @property
+    def submenu_name(self):
         """
-            Populate view for artist id
-            @param artist_id as int
+            Get submenu name
+            @return str
         """
-        self.__added = []
-        self.__artist = App().artists.get_name(artist_id)
-        providers = []
-        if get_network_available("SPOTIFY"):
-            providers.append(App().spotify)
-        if App().lastfm is not None and get_network_available("LASTFM"):
-            providers.append(App().lastfm)
-        if not providers:
-            providers = [ArtistProvider()]
-        self.__populate(providers)
+        return _("Similar artists")
 
 #######################
 # PRIVATE             #
 #######################
-    def __populate(self, providers):
-        """
-            Populate view with providers
-            @param providers as []
-        """
-        if providers:
-            provider = providers.pop(0)
-            App().task_helper.run(provider.get_artist_id,
-                                  self.__artist, self.__cancellable,
-                                  callback=(self.__on_get_artist_id,
-                                            providers, provider))
-        elif not self.__listbox.get_children() and\
-                not self.__cancellable.is_cancelled():
-            self.__stack.set_visible_child_name("no-results")
-            self.__spinner.stop()
-
     def __sort_func(self, row_a, row_b):
         """
             Sort rows
@@ -218,24 +196,20 @@ class SimilarsMenu(Gtk.Bin):
         else:
             return False
 
-    def __on_get_artist_id(self, artist_id, providers, provider):
+    def __on_map(self, widget):
         """
-            Get similars
-            @param artist_id as str
-            @param providers as []
-            @param provider as SpotifySearch/LastFM
+            Populate widget
+            @param widget as Gtk.Widget
         """
-        if artist_id is None:
-            if providers:
-                self.__populate(providers)
-            elif not self.__cancellable.is_cancelled():
-                self.__stack.set_visible_child_name("no-results")
-                self.__spinner.stop()
-        else:
-            App().task_helper.run(provider.get_similar_artists,
-                                  artist_id, self.__cancellable,
-                                  callback=(self.__on_similar_artists,
-                                            providers))
+        if self.__added:
+            return
+        from lollypop.similars import Similars
+        similars = Similars()
+        App().task_helper.run(
+            similars.get_similar_artists,
+            [self.__artist_id],
+            self.__cancellable,
+            callback=(self.__on_get_similar_artists,))
 
     def __on_unmap(self, widget):
         """
@@ -256,7 +230,6 @@ class SimilarsMenu(Gtk.Bin):
             popover.hide()
         artist_name = row.artist_name
         if row.storage_type == StorageType.EPHEMERAL:
-            App().settings.set_value("search-spotify", GLib.Variant("b", True))
             App().lookup_action("search").activate(
                 GLib.Variant("s", artist_name))
         else:
@@ -264,16 +237,18 @@ class SimilarsMenu(Gtk.Bin):
                 sql_escape(artist_name))
             App().window.container.show_view([Type.ARTISTS], [artist_id])
 
-    def __on_similar_artists(self, artists, providers):
+    def __on_get_similar_artists(self, artists):
         """
             Add artist to view
-            @param artists as [str]
-            @param providers as []
+            @param artists as [(str, str)]
         """
         if artists:
-            (spotify_id, artist, cover_uri) = artists.pop(0)
+            self.__stack.set_visible_child(self.__listbox)
+            self.__label.hide()
+            (artist, cover_uri) = artists.pop(0)
             if artist in self.__added:
-                GLib.idle_add(self.__on_similar_artists, artists, providers)
+                GLib.idle_add(self.__on_get_similar_artists, artists)
+                return
             self.__added.append(artist)
             artist_id = App().artists.get_id_for_escaped_string(
                 sql_escape(artist))
@@ -289,9 +264,6 @@ class SimilarsMenu(Gtk.Bin):
             if row is not None:
                 row.show()
                 self.__listbox.add(row)
-            GLib.idle_add(self.__on_similar_artists, artists, providers)
-        elif not self.__cancellable.is_cancelled():
-            if self.__listbox.get_children():
-                self.__stack.set_visible_child(self.__listbox)
-                self.__spinner.stop()
-            self.__populate(providers)
+            GLib.idle_add(self.__on_get_similar_artists, artists)
+        else:
+            self.__label.set_text(_("No results"))
