@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,7 @@ from gettext import gettext as _
 
 from lollypop.pop_devices import DevicesPopover
 from lollypop.define import App, Repeat, Type
-from lollypop.utils import popup_widget, emit_signal
+from lollypop.utils import popup_widget, emit_signal, get_network_available
 from lollypop.progressbar import ButtonProgressBar
 
 
@@ -59,8 +59,8 @@ class ToolbarEnd(Gtk.Bin):
         App().add_action(self.__shuffle_action)
         App().add_action(self.__repeat_action)
 
-        self.__shuffle_button = builder.get_object("shuffle_button")
-        self.__shuffle_button_image = builder.get_object(
+        self.__playback_button = builder.get_object("playback_button")
+        self.__playback_button_image = builder.get_object(
             "shuffle_button_image")
         shuffle_button_action = Gio.SimpleAction.new("shuffle-button", None)
         shuffle_button_action.connect("activate",
@@ -88,20 +88,21 @@ class ToolbarEnd(Gtk.Bin):
         App().add_action(scrobbling_action)
         App().set_accels_for_action("app.scrobbling", ["<Control><Shift>s"])
 
+        self.__menu_button = builder.get_object("menu_button")
         self.__home_button = builder.get_object("home_button")
+        self.__devices_button = builder.get_object("devices_button")
+        self.__playback_button = builder.get_object("playback_button")
         self.__set_shuffle_icon()
 
         button_progress_bar = ButtonProgressBar()
         overlay = builder.get_object("overlay")
         overlay.add_overlay(button_progress_bar)
         overlay.set_overlay_pass_through(button_progress_bar, True)
-        devices_button = builder.get_object("devices_button")
         self.__devices_popover = DevicesPopover(button_progress_bar)
         self.__devices_popover.connect(
-                "hidden", self.__on_menu_hidden, devices_button)
+                "hidden", self.__on_menu_hidden, self.__devices_button)
         self.__devices_popover.connect("content-changed",
-                                       self.__on_devices_content_changed,
-                                       devices_button)
+                                       self.__on_devices_content_changed)
         self.__devices_popover.populate()
         builder.connect_signals(self)
         window.connect("adaptive-changed", self.__on_adaptive_changed)
@@ -116,14 +117,6 @@ class ToolbarEnd(Gtk.Bin):
         """
         return self.__devices_popover
 
-    @property
-    def home_button(self):
-        """
-            Get home button
-            @return Gtk.Button
-        """
-        return self.__home_button
-
 #######################
 # PROTECTED           #
 #######################
@@ -147,7 +140,8 @@ class ToolbarEnd(Gtk.Bin):
             from lollypop.widgets_menu import MenuBuilder
             self.__playback_menu = MenuBuilder(self.__shuffle_menu)
             self.__playback_menu.show()
-            popover = popup_widget(self.__playback_menu, button)
+            popover = popup_widget(self.__playback_menu, button,
+                                   None, None, None)
             if popover is None:
                 self.__playback_menu.connect("hidden",
                                              self.__on_menu_hidden,
@@ -177,7 +171,7 @@ class ToolbarEnd(Gtk.Bin):
                 emit_signal(self.__playback_menu, "hidden", True)
             self.__app_menu = ApplicationMenu()
             self.__app_menu.show()
-            popover = popup_widget(self.__app_menu, button)
+            popover = popup_widget(self.__app_menu, button, None, None, None)
             if popover is None:
                 self.__app_menu.connect("hidden",
                                         self.__on_menu_hidden,
@@ -194,58 +188,77 @@ class ToolbarEnd(Gtk.Bin):
         """
             Init party submenu with current ids
         """
-        def on_change_state(action, value, genre_id):
-            action.set_state(value)
-            ids = list(App().settings.get_value("party-ids"))
-            genre_ids = App().genres.get_ids() + [Type.WEB]
-            # Select all
-            if genre_id is None:
-                # Update others
-                for genre_id in genre_ids:
-                    action = App().lookup_action("genre_%s" % genre_id)
-                    if action.get_state() != value:
-                        action.set_state(value)
-                ids = []
-            # Party id added
-            elif value:
-                ids.append(genre_id)
-            # Party id removed
-            elif ids and len(ids) > 1:
-                ids.remove(genre_id)
-            # Initial value
-            else:
-                ids = list(genre_ids)
-                ids.remove(genre_id)
-            App().settings.set_value("party-ids", GLib.Variant("ai", ids))
-            App().player.set_party_ids()
-            App().player.set_next()
+        def update_actions():
+            # Update state with a thread
+            party_ids = list(App().settings.get_value("party-ids"))
+            all_ids = App().genres.get_ids() + [Type.WEB]
+            all_selected = not party_ids or len(party_ids) == len(all_ids)
+            App().lookup_action("all_party_ids").set_state(
+                GLib.Variant.new_boolean(all_selected))
+            for genre_id in all_ids:
+                in_party_ids = not party_ids or genre_id in party_ids
+                action = App().lookup_action("genre_%s" % genre_id)
+                action.set_state(GLib.Variant.new_boolean(in_party_ids))
 
-        party_ids = App().settings.get_value("party-ids")
-        all_ids = App().genres.get_ids()
-        all_selected = len(set(all_ids) & set(party_ids)) == len(all_ids) or\
-            not party_ids
+        def update_party_ids(party_ids, value, genre_id):
+            all_ids = App().genres.get_ids() + [Type.WEB]
+            if value:
+                if genre_id not in party_ids:
+                    party_ids.append(genre_id)
+                if len(party_ids) == len(all_ids):
+                    party_ids = []
+            else:
+                if not party_ids:
+                    party_ids = all_ids
+                if genre_id in party_ids:
+                    party_ids.remove(genre_id)
+            return party_ids
+
+        def on_change_state(action, value, genre_id):
+            party_ids = list(App().settings.get_value("party-ids"))
+            all_ids = App().genres.get_ids() + [Type.WEB]
+            action.set_state(value)
+            if genre_id is None:
+                for genre_id in all_ids:
+                    action = App().lookup_action("genre_%s" % genre_id)
+                    action.set_state(value)
+                    party_ids = update_party_ids(party_ids, value, genre_id)
+            else:
+                party_ids = update_party_ids(party_ids, value, genre_id)
+            all_selected = not party_ids or len(party_ids) == len(all_ids)
+            App().lookup_action("all_party_ids").set_state(
+                GLib.Variant("b", all_selected))
+            App().settings.set_value("party-ids",
+                                     GLib.Variant("ai", party_ids))
+            App().task_helper.run(App().player.set_party_ids,
+                                  callback=(
+                                    lambda x: App().player.set_next(),))
+
+        # Create actions
         action = Gio.SimpleAction.new_stateful(
                     "all_party_ids",
                     None,
-                    GLib.Variant.new_boolean(all_selected))
+                    GLib.Variant.new_boolean(False))
         action.connect("change-state", on_change_state, None)
         App().add_action(action)
         item = Gio.MenuItem.new(_("All genres"), "app.all_party_ids")
         self.__party_submenu.append_item(item)
-        genres = App().artists.get()
-        genres.append((Type.WEB, _("Web"), _("Web")))
+        genres = App().genres.get()
+        genres += [(Type.WEB, _("Web"), _("Web"))]
         for (genre_id, name, sortname) in genres:
-            in_party_ids = not party_ids or genre_id in party_ids
             action_name = "genre_%s" % genre_id
             action = Gio.SimpleAction.new_stateful(
                 action_name,
                 None,
-                GLib.Variant.new_boolean(in_party_ids))
+                GLib.Variant.new_boolean(False))
             action.connect("change-state", on_change_state, genre_id)
+            if genre_id == Type.WEB:
+                action.set_enabled(get_network_available("YOUTUBE"))
             App().add_action(action)
             menu_str = name if len(name) < 20 else name[0:20] + "…"
             item = Gio.MenuItem.new(menu_str, "app.%s" % action_name)
             self.__party_submenu.append_item(item)
+        App().task_helper.run(update_actions)
 
     def __set_shuffle_icon(self):
         """
@@ -254,21 +267,21 @@ class ToolbarEnd(Gtk.Bin):
         shuffle = App().settings.get_value("shuffle")
         repeat = App().settings.get_enum("repeat")
         if shuffle:
-            self.__shuffle_button_image.set_from_icon_name(
+            self.__playback_button_image.set_from_icon_name(
                 "media-playlist-shuffle-symbolic",
                 Gtk.IconSize.BUTTON)
         elif repeat == Repeat.TRACK:
-            self.__shuffle_button_image.get_style_context().remove_class(
+            self.__playback_button_image.get_style_context().remove_class(
                 "selected")
-            self.__shuffle_button_image.set_from_icon_name(
+            self.__playback_button_image.set_from_icon_name(
                 "media-playlist-repeat-song-symbolic",
                 Gtk.IconSize.BUTTON)
         elif repeat == Repeat.ALL:
-            self.__shuffle_button_image.set_from_icon_name(
+            self.__playback_button_image.set_from_icon_name(
                 "media-playlist-repeat-symbolic",
                 Gtk.IconSize.BUTTON)
         else:
-            self.__shuffle_button_image.set_from_icon_name(
+            self.__playback_button_image.set_from_icon_name(
                 "media-playlist-consecutive-symbolic",
                 Gtk.IconSize.BUTTON)
 
@@ -321,8 +334,8 @@ class ToolbarEnd(Gtk.Bin):
             @param action as Gio.SimpleAction
             @param param as GLib.Variant
         """
-        self.__shuffle_button.set_active(
-            not self.__shuffle_button.get_active())
+        self.__playback_button.set_active(
+            not self.__playback_button.get_active())
 
     def __on_repeat_changed(self, settings, value):
         """
@@ -343,17 +356,16 @@ class ToolbarEnd(Gtk.Bin):
         self.__playback_menu = None
         button.set_active(False)
 
-    def __on_devices_content_changed(self, popover, count, devices_button):
+    def __on_devices_content_changed(self, popover, count):
         """
             Show/Hide device button
             @param popover as DevicesPopover
             @param count as int
-            @param devices_button as Gtk.ToggleButton
         """
         if count:
-            devices_button.show()
+            self.__devices_button.show()
         else:
-            devices_button.hide()
+            self.__devices_button.hide()
 
     def __on_adaptive_changed(self, window, status):
         """
@@ -361,7 +373,9 @@ class ToolbarEnd(Gtk.Bin):
             @param window as Window
             @param status as bool
         """
-        if status:
+        mini_revealed = App().window.miniplayer is not None and\
+            App().window.miniplayer.revealed
+        if status and not mini_revealed:
             self.__home_button.show()
         else:
             self.__home_button.hide()
