@@ -10,21 +10,38 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk
 
 from lollypop.utils import popup_widget
 from lollypop.view_lazyloading import LazyLoadingView
 from lollypop.define import App, ViewType, MARGIN, StorageType
-from lollypop.controller_view import ViewController, ViewControllerType
 from lollypop.widgets_row_album import AlbumRow
 from lollypop.helper_gestures import GesturesHelper
+from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
+class ListBox(Gtk.ListBox):
+    def __init__(self):
+        Gtk.ListBox.__init__(self)
+        self.__width = 0
+
+    def set_width(self, width):
+        self.set_property("halign", Gtk.Align.CENTER)
+        self.__width = width
+
+    def do_get_preferred_width(self):
+        if self.__width == 0:
+            return Gtk.ListBox.do_get_preferred_width(self)
+        else:
+            return (self.__width, self.__width)
+
+
+class AlbumsListView(LazyLoadingView, SignalsHelper, GesturesHelper):
     """
         View showing albums
     """
 
+    @signals_map
     def __init__(self, genre_ids, artist_ids, view_type):
         """
             Init widget
@@ -33,15 +50,13 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
             @param view_type as ViewType
         """
         LazyLoadingView.__init__(self, StorageType.ALL, view_type)
-        ViewController.__init__(self, ViewControllerType.ALBUM)
-        self.__width = 0
         self.__genre_ids = genre_ids
         self.__artist_ids = artist_ids
         self.__reveals = []
         # Calculate default album height based on current pango context
         # We may need to listen to screen changes
         self.__height = AlbumRow.get_best_height(self)
-        self._box = Gtk.ListBox()
+        self._box = ListBox()
         self._box.set_margin_bottom(MARGIN)
         self._box.set_margin_end(MARGIN)
         self._box.get_style_context().add_class("trackswidget")
@@ -53,7 +68,11 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
             from lollypop.helper_dnd import DNDHelper
             self.__dnd_helper = DNDHelper(self._box, view_type)
             self.__dnd_helper.connect("dnd-insert", self.__on_dnd_insert)
-        self.add_widget(self._box)
+        return [
+            (App().player, "current-changed", "_on_current_changed"),
+            (App().player, "duration-changed", "_on_duration_changed"),
+            (App().art, "album-artwork-changed", "_on_artwork_changed")
+        ]
 
     def add_reveal_albums(self, albums):
         """
@@ -62,21 +81,19 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
         """
         self.__reveals += list(albums)
 
-    def insert_album(self, album, index):
+    def add_value(self, album):
         """
-            Insert album at index
+            Insert item
             @param album as Album
-            @param index as int
         """
-        children = self.children
-        if children and\
-                index < len(children) + 1 and\
-                children[index].album.id == album.id:
-            children[index].tracks_view.append_rows(album.tracks)
+        # Merge album if previous is same
+        if self.children and self.children[-1].album.id == album.id:
+            track_ids = self.children[-1].album.track_ids
+            for track in album.tracks:
+                if track.id not in track_ids:
+                    self.children[-1].tracks_view.append_row(track)
         else:
-            row = AlbumRow(album, self.__height, self.view_type)
-            row.populate()
-            self.insert_row(row, index)
+            LazyLoadingView.populate(self, [album])
 
     def insert_row(self, row, index):
         """
@@ -84,7 +101,9 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
             @param row as AlbumRow
             @param index as int
         """
-        row.connect("activated", self.__on_row_activated)
+        row.connect("activated", self._on_row_activated)
+        row.connect("destroy", self._on_row_destroy)
+        row.connect("track-removed", self._on_track_removed)
         row.show()
         self._box.insert(row, index)
 
@@ -94,9 +113,18 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
             @param albums as [Album]
         """
         for child in self._box.get_children():
-            GLib.idle_add(child.destroy)
+            self._box.remove(child)
         LazyLoadingView.populate(self, albums)
 
+    def clear(self):
+        """
+            Clear the view
+        """
+        self.stop()
+        self.__reveals = []
+        for child in self._box.get_children():
+            self._box.remove(child)
+
     def jump_to_current(self):
         """
             Scroll to album
@@ -105,29 +133,6 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
         if y is not None:
             self.scrolled.get_vadjustment().set_value(y)
 
-    def clear(self, clear_albums=False):
-        """
-            Clear the view
-        """
-        for child in self._box.get_children():
-            GLib.idle_add(child.destroy)
-        if clear_albums:
-            App().player.clear_albums()
-
-    def set_width(self, width):
-        """
-            Set list width
-            @param width as int
-        """
-        self.set_property("halign", Gtk.Align.CENTER)
-        self.__width = width
-
-    def do_get_preferred_width(self):
-        if self.__width == 0:
-            return LazyLoadingView.do_get_preferred_width(self)
-        else:
-            return (self.__width, self.__width)
-
     @property
     def args(self):
         """
@@ -175,7 +180,9 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
         if self.destroyed:
             return None
         row = AlbumRow(album, self.__height, self.view_type)
-        row.connect("activated", self.__on_row_activated)
+        row.connect("activated", self._on_row_activated)
+        row.connect("destroy", self._on_row_destroy)
+        row.connect("track-removed", self._on_track_removed)
         row.show()
         self._box.add(row)
         return row
@@ -248,31 +255,46 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
         else:
             LazyLoadingView._on_populated(self, widget)
 
+    def _on_row_activated(self, row, track):
+        pass
+
+    def _on_row_destroy(self, row):
+        pass
+
+    def _on_track_removed(self, row, track):
+        pass
+
 #######################
 # PRIVATE             #
 #######################
+    def __get_current_ordinate(self):
+        """
+            If current track in widget, return it ordinate,
+            @return y as int
+        """
+        y = None
+        for child in self._box.get_children():
+            if child.album == App().player.current_track.album:
+                child.reveal(True)
+                y = child.translate_coordinates(self._box, 0, 0)[1]
+        return y
+
     def __popup_menu(self, x, y):
         """
             Popup menu for album
             @param x as int
             @param y as int
         """
-        def on_hidden(popover, hide, row):
-            row.unset_state_flags(Gtk.StateFlags.CHECKED)
-
         row = self._box.get_row_at_y(y)
         if row is None:
             return
         from lollypop.menu_objects import AlbumMenu
         from lollypop.widgets_menu import MenuBuilder
-        menu = AlbumMenu(row.album, ViewType.ALBUM,
+        menu = AlbumMenu(row.album, ViewType.ALBUM, row.album.storage_type,
                          App().window.is_adaptive)
         menu_widget = MenuBuilder(menu)
         menu_widget.show()
-        popover = popup_widget(menu_widget, self._box, x, y)
-        if popover is not None:
-            popover.connect("hidden", on_hidden, row)
-        row.set_state_flags(Gtk.StateFlags.CHECKED, True)
+        popup_widget(menu_widget, self._box, x, y, row)
 
     def __reveal_row(self, row):
         """
@@ -282,18 +304,6 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
         if style_context.has_class("drag-down"):
             row.reveal(True)
 
-    def __get_current_ordinate(self):
-        """
-            If current track in widget, return it ordinate,
-            @return y as int
-        """
-        y = None
-        for child in self._box.get_children():
-            if child.album.id == App().player.current_track.album.id:
-                child.reveal(True)
-                y = child.translate_coordinates(self._box, 0, 0)[1]
-        return y
-
     def __on_dnd_insert(self, dnd_helper, row, index):
         """
             Insert row at index
@@ -302,22 +312,3 @@ class AlbumsListView(LazyLoadingView, ViewController, GesturesHelper):
             @param index as int
         """
         self.insert_row(row, index)
-
-    def __on_row_activated(self, row, track):
-        """
-            Start playback
-            @param row as AlbumRow
-            @param track_id as int
-        """
-        # In party mode, just play track_id and continue party mode
-        if App().player.is_party or self.view_type & ViewType.PLAYBACK:
-            App().player.load(track)
-        elif self.view_type & ViewType.PLAYLISTS:
-            albums = []
-            for album_row in self.children:
-                albums.append(album_row.album)
-            App().player.play_track_for_albums(track, albums)
-        else:
-            App().player.remove_album_by_id(track.album.id)
-            App().player.add_album(track.album)
-            App().player.load(track)
