@@ -13,8 +13,9 @@
 from locale import strcoll
 
 from lollypop.view_flowbox import FlowBoxView
-from lollypop.define import App, Type, ViewType
-from lollypop.utils import popup_widget, tracks_to_albums
+from lollypop.define import App, Type, ViewType, StorageType
+from lollypop.utils import popup_widget
+from lollypop.utils_album import tracks_to_albums
 from lollypop.objects_track import Track
 from lollypop.widgets_playlist_rounded import PlaylistRoundedWidget
 from lollypop.widgets_banner_playlists import PlaylistsBannerWidget
@@ -33,8 +34,8 @@ class PlaylistsManagerView(FlowBoxView, SignalsHelper):
             Init decade view
             @param view_type as ViewType
         """
-        FlowBoxView.__init__(self, view_type |
-                             ViewType.SCROLLED | ViewType.OVERLAY)
+        FlowBoxView.__init__(self, StorageType.ALL,
+                             view_type | ViewType.OVERLAY)
         self.__signal_id = None
         self._empty_icon_name = "emblem-documents-symbolic"
         if self.args is None:
@@ -44,7 +45,9 @@ class PlaylistsManagerView(FlowBoxView, SignalsHelper):
             self.__banner.show()
             self.add_widget(self._box, self.__banner)
         return [
-            (App().playlists, "playlists-changed", "_on_playlist_changed")
+            (App().playlists, "playlists-added", "_on_playlist_added"),
+            (App().playlists, "playlists-removed", "_on_playlist_removed"),
+            (App().playlists, "playlists-renamed", "_on_playlist_renamed")
         ]
 
     def populate(self):
@@ -83,7 +86,7 @@ class PlaylistsManagerView(FlowBoxView, SignalsHelper):
         # Compatibility with SelectionList
         if isinstance(value, tuple):
             value = value[0]
-        widget = PlaylistRoundedWidget(value, self._view_type,
+        widget = PlaylistRoundedWidget(value, self.view_type,
                                        self.font_height)
         self._box.insert(widget, -1)
         widget.show()
@@ -158,28 +161,39 @@ class PlaylistsManagerView(FlowBoxView, SignalsHelper):
             return
         self.__popup_menu(child)
 
-    def _on_playlist_changed(self, playlists, playlist_id):
+    def _on_playlist_added(self, playlists, playlist_id):
         """
-            Update view based on playlist_id status
+            Add playlist
             @param playlists as Playlists
             @param playlist_id as int
         """
-        exists = playlists.exists(playlist_id)
-        if exists:
-            item = None
-            for child in self._box.get_children():
-                if child.data == playlist_id:
-                    item = child
-                    break
-            if item is None:
-                self.add_value(playlist_id)
-            else:
-                name = App().playlists.get_name(playlist_id)
-                item.rename(name)
-        else:
-            for child in self._box.get_children():
-                if child.data == playlist_id:
-                    child.destroy()
+        self.add_value(playlist_id)
+
+    def _on_playlist_removed(self, playlists, playlist_id):
+        """
+            Remove playlist
+            @param playlists as Playlists
+            @param playlist_id as int
+        """
+        for child in self._box.get_children():
+            if child.data == playlist_id:
+                child.destroy()
+                break
+
+    def _on_playlist_renamed(self, playlists, playlist_id):
+        """
+            Rename playlist
+            @param playlists as Playlists
+            @param playlist_id as int
+        """
+        item = None
+        for child in self._box.get_children():
+            if child.data == playlist_id:
+                item = child
+                break
+        if item is not None:
+            name = App().playlists.get_name(playlist_id)
+            item.rename(name)
 
 #######################
 # PRIVATE             #
@@ -191,7 +205,8 @@ class PlaylistsManagerView(FlowBoxView, SignalsHelper):
         """
         from lollypop.widgets_menu import MenuBuilder
         from lollypop.menu_playlist import PlaylistMenu, PlaylistMenuExt
-        menu = PlaylistMenu(child.data, App().window.is_adaptive)
+        menu = PlaylistMenu(child.data, self.view_type,
+                            App().window.is_adaptive)
         menu_widget = MenuBuilder(menu)
         if child.data >= 0:
             menu_widget = MenuBuilder(menu)
@@ -201,7 +216,7 @@ class PlaylistsManagerView(FlowBoxView, SignalsHelper):
         else:
             menu_widget = MenuBuilder(menu)
         menu_widget.show()
-        popup_widget(menu_widget, child)
+        popup_widget(menu_widget, child, None, None, None)
 
 
 class PlaylistsManagerDeviceView(PlaylistsManagerView):
